#coding:utf-8
"""
Created on 17.09.14

@author: kasimova
"""
from m3.actions import ApplicationLogicException
import objectpack

from kinder.core.audit_log.helpers import get_model, get_field
from concentrator.models import UpdateParams
import forms


class UpdateConfigPack(objectpack.ObjectPack):
    __PERMISSION_NS = "kinder.plugins.concentrator.update_conf.actions.UpdateConfigPack"
    title = u'Параметры для изменения данных через ЕПГУ'
    model = UpdateParams
    edit_window = add_window = forms.EditWindow
    need_check_permission = True
    # list_sort_order = ['model_name_verbose']
    columns = [
        {
            'header': u'Наименование модели',
            'data_index': 'model_name',
            'sortable': True,
        },
        {
            'header': u'Наименование поля',
            'data_index': 'field_name',
            'width': 50,
        },

    ]

    def __init__(self):
        super(UpdateConfigPack, self).__init__()
        self.replace_action('list_window_action',
                            UpdateConfigListWindowAction())
        self.sub_permissions = super(
            UpdateConfigPack, self
        ).sub_permissions.copy()
        self.sub_permissions[self.list_window_action.perm_code] = \
            self.list_window_action.verbose_name

    def prepare_row(self, obj, request, context):
        """
        Подменяем на verbose_name
        @param obj:
        @param request:
        @param context:
        @return:
        """
        obj = super(UpdateConfigPack, self).prepare_row(obj, request, context)
        model_cls = get_model(obj.model_name)
        if hasattr(model_cls._meta, 'verbose_name'):
            obj.model_name = get_model(obj.model_name)._meta.verbose_name

        field = get_field(model_cls, obj.field_name)
        if field and hasattr(field, 'verbose_name'):
            obj.field_name = field.verbose_name
        return obj

    def save_row(self, obj, create_new, request, context):
        # заглушка для 1.0.х версии. в 1.1.х от дублей
        # избавились путем добавления уникального ключа
        is_double = UpdateParams.objects.filter(
            field_name=obj.field_name, model_name=obj.model_name).exists()
        if not is_double:
            super(UpdateConfigPack, self).save_row(obj, create_new, request, context)
        else:
            raise ApplicationLogicException(
                u'Поле для данной модели уже добавлено в справочник.')

    def extend_menu(self, desk):
        return desk.SubMenu(
            u'Справочники',
            desk.Item(name=self.title, pack=self.list_window_action),
            icon='menu-dicts-16',
        )


class UpdateConfigListWindowAction(objectpack.ObjectListWindowAction):
    __PERMISSION_NS = "kinder.plugins.concentrator.update_conf.actions.UpdateConfigListWindowAction"
    need_check_permission = True
    verbose_name = u'Просмотр и редактирование'