# -*- coding: utf-8 -*-

# Standard Library

# Vendor Library
from spyne.model.primitive import UnsignedInteger

# Project Library
from kinder.webservice.spyne_ws.declaration_info.types import (
    CustomComplexModel)
from concentrator.webservice.entities import *
from lipetsk_specifics.models import PrivilegeOwnerEnum


class LipetskEduOrganization(EduOrganization):
    """
    Информация о учреждении
    """
    CodeMO = Integer
    OkatoMO = Unicode


class LipetskEduOrganizationsData(ComplexModel):
    """
    Информация о ОО
    """
    EduOrganization = LipetskEduOrganization.customize(max_occurs='unbounded')
    AllowOfferOther = Boolean

    class Annotations(ComplexModel.Annotations):
        doc = u'Информация об ОО'


class WhoHaveBenefit(ComplexModel):
    Type = UnsignedInteger(values=PrivilegeOwnerEnum.values.keys(),
                           min_occurs=1, nillable=False)
    FirstName = FirstName
    LastName = LastName
    MiddleName = MiddleName
    DocType = DocType
    DocSeria = DocSeria
    DocNumber = DocNumber
    DocIssueDate = DocIssueDate
    DocIssuerName = DocIssuerName
    Snils = Snils

    class Annotations(ComplexModel.Annotations):
        doc = u'Информация об ОО'


class LipetskBenefits(ComplexModel):
    Benefit = BenefitItem.customize(min_occurs=0, max_occurs="unbounded",
                                    nillable=False)
    WhoHaveBenefit = WhoHaveBenefit.customize(min_occurs=1, nillable=False)

    BenefitsDocInfo = NotBlankUnicode
    Other = NotBlankUnicode


# -----------------------------------------------------------------------------
# типы сервиса Создания заявления
# -----------------------------------------------------------------------------
class LipetskApplicantData(ApplicantData):
    """
    Информация о заявителе
    добавили AddressResidence AddressRegistration
    """
    AddressRegistration = AddressRegistration
    AddressResidence = AddressResidence

    class Annotations(ComplexModel.Annotations):
        doc = u'Контейнер, содержит информацию о заявителе'


class LipetskDeclaredPersonData(ComplexModel):
    """
    Информация о ребенке
    добавили DateOfActNumber
    """
    FirstName = FirstName(min_occurs=1)
    LastName = LastName(min_occurs=1)
    MiddleName = MiddleName
    Snils = Snils(min_occurs=1)
    BirthPlace = Unicode
    BirthDocSeria = DocSeria
    BirthDocNumber = DocNumber
    BirthDocActNumber = Unicode.customize(min_occurs=0)
    BirthDocIssueDate = DocIssueDate
    BirthDocIssuer = Unicode.customize(min_occurs=0)
    BirthDocForeign = Unicode.customize(min_occurs=0)
    BirthDocForeignNumber = Unicode.customize(min_occurs=0)
    AgeGroupType = AgeGroupType
    DateOfBirth = Date
    # В большом городе
    Sex = Sex
    AddressRegistration = AddressRegistration
    AddressResidence = AddressResidence
    Benefits = LipetskBenefits.customize(min_occurs=0, nillable=False)
    DateOfActNumber = Date
    ArrivalTimeType = Unicode
    IssuerState = Unicode

    class Annotations(ComplexModel.Annotations):
        doc = u'Контейнер, содержит информацию о ребенке'


class NewApplicationRequest(ComplexModel):
    _type_info = [
        ('ExternalId', ExternalId),
        ('SubmitDate', SubmitDate),
        ('EntryDate', EntryDate),
        ('EducationProgramType', EducationProgramType),
        ('AdaptationProgramType', AdaptationProgramType),
        ('Applicant', LipetskApplicantData),
        ('DeclaredPerson', LipetskDeclaredPersonData),
        ('EduOrganizations', EduOrganizationsData),
        ('DocumentReferences', DocumentReferencesData.customize(
            min_occurs=0))
    ]


# -----------------------------------------------------------------------------
# типы сервиса Изменения данных заявки
# -----------------------------------------------------------------------------
class UpdateApplicationRequest(ComplexModel):

    ExternalId = ExternalId
    State = StateCode
    SubmitDate = SubmitDate
    EntryDate = EntryDate
    EducationProgramType = EducationProgramType
    AdaptationProgramType = AdaptationProgramType
    Applicant = LipetskApplicantData.customize(nillable=False, min_occurs=1)
    DeclaredPerson = LipetskDeclaredPersonData.customize(
        nillable=False, min_occurs=1)
    EduOrganizationsData = EduOrganizationsData.customize(nillable=False,
                                                          min_occurs=1)
    DocumentReferences = DocumentReferencesData.customize(
        min_occurs=0)


# -----------------------------------------------------------------------------
# типы сервиса Получение данных Заявления для изменения
# -----------------------------------------------------------------------------
class GetApplicationResponse(ComplexModel):
    SubmitDate = SubmitDate
    EntryDate = EntryDate
    EducationProgramType = EducationProgramType
    AdaptationProgramType = AdaptationProgramType
    Applicant = LipetskApplicantData.customize(nillable=False, min_occurs=1)
    DeclaredPerson = LipetskDeclaredPersonData.customize(
        nillable=False, min_occurs=1)
    EduOrganizationsData = LipetskEduOrganizationsData.customize(
        nillable=False, min_occurs=1)
    ApplicationRules = ApplicationRulesData.customize(nillable=False,
                                                      min_occurs=1)
    DocumentReferences = DocumentReferencesData.customize(min_occurs=0)


class DeclarationStatus(CustomComplexModel):

    DeclID = Unicode
    AllCategoryPosition = Unicode
    StatusName = Unicode(nillable=False, min_occurs=1)
    Position = Unicode(nillable=False, min_occurs=1)
    CommonPosition = Unicode
    CommonRayonPosition = Unicode
    SummaryRayonPosition = Unicode
    CommonMoPosition = Unicode
    SummaryMoPosition = Unicode
    UnitName = Unicode(nillable=False, min_occurs=1)
    Date = Unicode(nillable=False, min_occurs=1)
    DesiredDate = Unicode(nillable=False, min_occurs=1)
    StatusDescription = Unicode(nillable=False, min_occurs=1)
    Raion = Unicode()

    class Annotations(CustomComplexModel.Annotations):
        doc = u'Информация о статусе заявления и позиции в очереди'
